//+------------------------------------------------------------------+
//|                                                                  |                                                 VQ.mq4 |
//|                                               Volatility Quality |
//|                                                by raff1410@o2.pl |
//+------------------------------------------------------------------+

#property  indicator_chart_window
#property  indicator_buffers 2
#property  indicator_color1  Green
#property  indicator_color2  Red
#property  indicator_width1  2
#property  indicator_width2  2

extern string TimeFrame  = "Current time frame";
extern int    Length = 5;
extern int    Method = 3;
extern int    Smoothing = 1;
extern int    Filter = 5;
extern bool   Steady  = false;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsNotification = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;

double   VQ[];
double   SumVQ[];
double   DIR[];
double   UpBuffer[];
double   DnBuffer[];


int      timeFrame;
bool     calculateValue;
bool     returnBars;
string   indicatorFileName;

//+------------------------------------------------------------------+

int init()
{
IndicatorBuffers(5);
SetIndexStyle(0,DRAW_HISTOGRAM);
SetIndexBuffer(0,UpBuffer);
SetIndexStyle(1,DRAW_HISTOGRAM);
SetIndexBuffer(1,DnBuffer);
SetIndexBuffer(2,SumVQ);
SetIndexBuffer(3,VQ);
SetIndexBuffer(4,DIR);
SetIndexEmptyValue(0, 0.0);
SetIndexEmptyValue(1, 0.0);
SetIndexEmptyValue(2, 0.0);
SetIndexEmptyValue(3, 0.0);

if (Length < 2) Length = 2;
if (Method < 0) Method = 0;
if (Method > 3) Method = 3;
if (Smoothing < 0) Smoothing = 0;
if (Filter < 0) Filter = 0;

         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

string short_name = timeFrameToString(timeFrame)+" VQ | "+ Length + " , " + 
Method + " , " + Smoothing + " , " + Filter + "  | ";
IndicatorShortName(short_name);

return(0);
}

//+------------------------------------------------------------------+

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { UpBuffer[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {
      for(int i=limit; i>=0; i--)
      {
         double MH  = iMA(NULL,0,Length,0,Method,PRICE_HIGH,i);
         double ML  = iMA(NULL,0,Length,0,Method,PRICE_LOW,i);
         double MO  = iMA(NULL,0,Length,0,Method,PRICE_OPEN,i);
         double MC  = iMA(NULL,0,Length,0,Method,PRICE_CLOSE,i);
         double MC1 = iMA(NULL,0,Length,0,Method,PRICE_CLOSE,i+Smoothing);
         if (Steady==true) 
         {
            MC  = iMA(NULL,0,Length,0,Method,PRICE_MEDIAN,i); 
            MC1 = iMA(NULL,0,Length,0,Method,PRICE_MEDIAN,i+Smoothing);
         }

         if((MH - ML)>0) VQ[i] = MathAbs(((MC - MC1) / MathMax(MH - ML, MathMax(MH - MC1, MC1 - ML)) + (MC - MO) / (MH - ML)) * 0.5) * ((MC - MC1 + (MC - MO)) * 0.5);
         SumVQ[i] = SumVQ[i + 1] + VQ[i];
         
         if (Filter > 0) if (MathAbs(SumVQ[i] - SumVQ[i + 1]) < Filter * Point) SumVQ[i] = SumVQ[i + 1];
         DIR[i] = DIR[i+1];
            if (SumVQ[i] > SumVQ[i+1]) DIR[i] = 1;
            if (SumVQ[i] < SumVQ[i+1]) DIR[i] = -1;
            if (DIR[i] > 0)
            {
               UpBuffer[i] = High[i];
               DnBuffer[i] = Low[i];
            }
            if (DIR[i] < 0)
            {
               DnBuffer[i] = High[i];
               UpBuffer[i] = Low[i];
            }
      }
      manageAlerts();
      return(0);
   }   

   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for ( i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         DIR[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",Length,Method,Smoothing,Filter,Steady,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,4,y);
            if (DIR[i] > 0)
            {
               UpBuffer[i] = High[i];
               DnBuffer[i] = Low[i];
            }
            if (DIR[i] < 0)
            {
               DnBuffer[i] = High[i];
               UpBuffer[i] = Low[i];
            }
   }
   return(0);         
   
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
return(s);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static datetime prevTime1  = 0;
      static string   prevAlert1 = "";
         if (DIR[whichBar] != DIR[whichBar+1])
         {
            if (DIR[whichBar] ==  1) doAlert(prevTime1,prevAlert1," VQ changed dirrection to up");
            if (DIR[whichBar] == -1) doAlert(prevTime1,prevAlert1," VQ changed dirrection to down");
         }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message = timeFrameToString(Period())+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
          if (alertsMessage)      Alert(message);
          if (alertsNotification) SendNotification(message);
          if (alertsEmail)        SendMail(StringConcatenate(Symbol(),"VQ 0.5 "),message);
          if (alertsSound)        PlaySound("alert2.wav");
   }
}